#!/usr/bin/perl -w
# ===============================================
# (C) 2017 AO Kaspersky Lab. All Rights Reserved.
# ===============================================

BEGIN { unshift @INC, $1 if $0 =~ /(.*)[\/]/; }

use strict;
use klinstall;
use appdata;

my $INSTALL_MSG_1 = <<INSTALL_MSG_1_END;
%1 cannot be installed on this system: %2.
INSTALL_MSG_1_END

my $INSTALL_MSG_2 = <<INSTALL_MSG_2_END;
%1 was not tested under this Linux distribution.
INSTALL_MSG_2_END

my $INSTALL_MSG_3 = <<INSTALL_MSG_3_END;
%1 was not tested under this version of %2.
INSTALL_MSG_3_END

my $INSTALL_MSG_4 = <<INSTALL_MSG_4_END;
%1 version %2 is already installed.
INSTALL_MSG_4_END

my $INSTALL_MSG_5 = <<INSTALL_MSG_5_END;
%1 is not compatible with already installed %2 because of %3.

You should uninstall %2 before installing %1.
INSTALL_MSG_5_END

my $INSTALL_MSG_6 = <<INSTALL_MSG_6_END;
%1 requires %2 version %3 or greater to work.
INSTALL_MSG_6_END

my $INSTALL_MSG_7 = <<INSTALL_MSG_7_END;
%1 needs %2 to perform %3.
INSTALL_MSG_7_END

my $INSTALL_MSG_16 = <<INSTALL_MSG_16_END;

%1 is installed.
Binaries were installed in %2

INSTALL_MSG_16_END

my $INSTALL_MSG_18 = <<INSTALL_MSG_18_END;
%1 has been installed successfully but needs to be properly configured before using.
Please run <B>%2</B> script by yourself to configure installed software.
INSTALL_MSG_18_END

sub sigint_handler
{
	print STDERR "Interrupted.\n";
	exit 0;
}




	my %cmdline_selected;
	my %cmdline_options_list =
			("debug" => "Enable debug mode",
			 "force" => "Force installation even if there's a conflict",
			 "upgrade" => "Perform upgrade",
			 "help"  => "This help message");

	$SIG{'INT'} = "sigint_handler";

	klinstall::check_for_root ("installation");

	my ($logic, $pkgmgr) = klinstall::get_work_logic($0, \@ARGV);

		my @argslist = @ARGV;
		push @argslist, split ('\s+', $ENV{"KLINSTALL"}) if $ENV{"KLINSTALL"};

		foreach (@argslist)
		{
			next if !$_;

			my $arg = $_;
			$arg = $1 if $arg =~ /^--(.*)$/;

			if ( !$cmdline_options_list{$arg} && !$logic)
			{
				print "Unsupported command line option: $_\n\n";
				display_cmdline_options(%cmdline_options_list);
				exit 1;
			}

			$cmdline_selected{$arg} = 1;
		}

		if ( $cmdline_selected{"help"} && !$logic )
		{
			display_cmdline_options();
			exit 1;
		}

	if ( !$logic || $logic eq "pre" )
	{
		my %APPINFO = appdata::get_application_info();

		my ($arch, $name, $rev) = klinstall::get_os_info();

		klinstall::debug "Got info: $arch, os: $name, ver: $rev"
			if $cmdline_selected{"debug"};

		my $os_ver_reason = appdata::app_check_os_ver ($arch, $name, $rev);
		klinstall::fatal ($INSTALL_MSG_1, $APPINFO{"NAME"}, $os_ver_reason)
			if $os_ver_reason;

		my %linux_distro = klinstall::get_linux_info();
		if ( $linux_distro{"NAME"} )
		{
			my $res = appdata::app_check_linux_distro (lc $linux_distro{"NAME"}, $linux_distro{"VER"});

			$res == 1 and klinstall::warning ($INSTALL_MSG_2, $APPINFO{"NAME"})
				or $res == 2 and klinstall::warning ($INSTALL_MSG_3, $APPINFO{"NAME"}, $linux_distro{"NAME"})
		}

		my @req_comps = appdata::app_check_required_components();
		klinstall::fatal ($INSTALL_MSG_6, $APPINFO{"NAME"}, $req_comps[0]{"NAME"}, $req_comps[0]{"VER"})
			if @req_comps;

		my @opt_comps = appdata::app_check_optional_components();
		foreach (@opt_comps)
		{
			my %rec = %{$_};
			klinstall::warning ($INSTALL_MSG_7, $APPINFO{"NAME"}, $req_comps[0]{"NAME"}, $req_comps[0]{"WHAT2DO"});
		}

		if ( !$pkgmgr && !$cmdline_selected{"force"} )
		{
			my %already_installed = klinstall::read_app_info ($APPINFO{"ID"});
			klinstall::fatal ($INSTALL_MSG_4, $APPINFO{"NAME"}, $already_installed{"VERSION"}) if %already_installed;

			my %applist = appdata::app_get_conflict_ids ();
			foreach (keys %applist)
			{
				my %ai = klinstall::read_app_info ($_);
				next if !%ai;

				klinstall::fatal ($INSTALL_MSG_5, $APPINFO{"NAME"}, $ai{"NAME"}, $applist{"$_"});
			}
		}
	}

	my (@installed_files, @installed_dirs, %uniqmatch);

	if ( !$logic )
	{
		my %APPINFO = appdata::get_application_info();

		foreach (split ("\n", $APPINFO{"DIRS"}))
		{
			next if !/^(\d+)\s+([\w\:]+)\s+(.*)$/;
			my ($mode, $own, $dest) = ($1, $2, $3);
			my $realdest = $dest;

			if ( $dest =~ /^(\w+)\/(.*)$/ )
			{
				if ( !defined $APPINFO{$1} )
				{
					klinstall::warning ("Bug in installer script: file $2 should be installed in unknown directory $1.");
					next;
				}

				$realdest = defined $2 && length $2 ? "$APPINFO{$1}/$2" : "$APPINFO{$1}";
			}

			next if -d $realdest;
			push @installed_dirs, $dest;

			klinstall::output ("Creating directory $realdest\n");
			klinstall::install_path ($realdest, $mode, $own, \@installed_dirs);
		}

		foreach (split ("\n", $APPINFO{"FILES"}))
		{
			next if !/^(\d+)\s+([\w\:]+)\s+(.*?)\s+(.*)$/;
			my ($mode, $own, $dest, $src) = ($1, $2, $3, $4);
			my $realdest = $dest;

			if ( $dest =~ /^(\w+)\/(.*)$/ )
			{
				if ( !defined $APPINFO{$1} )
				{
					klinstall::warning ("Bug in installer script: file $2 should be installed in unknown directory $1.");
					next;
				}

				$realdest = "$APPINFO{$1}/$2";
			}

			if ( $realdest =~ /(.*)\/(.*?)$/ && !-d $1 )
			{
				my $msg = klinstall::make_path ($1, 0755, \@installed_dirs);
				klinstall::fatal ($msg) if $msg;
			}

			push @installed_files, $dest;
			klinstall::output ("Installing file $realdest\n");
			klinstall::install_file ($src, $realdest, $mode, $own);
		}

		%uniqmatch = ();
		@installed_files = grep { ! $uniqmatch{$_} ++ } @installed_files;
		%uniqmatch = ();
		@installed_dirs = grep { ! $uniqmatch{$_} ++ } @installed_dirs;

		print ("\n");
	}

	if ( !$logic || $logic eq "post" )
	{
		my %APPINFO = appdata::get_application_info();
		my %INSTINFO = klinstall::read_app_info ($APPINFO{"ID"});

		my %stored_info;
		$stored_info{"ID"} = $APPINFO{"ID"};
		$stored_info{"NAME"} = $APPINFO{"NAME"};
		$stored_info{"VERSION"} = $APPINFO{"VER"};
		$stored_info{"USED_PKGMGR"} = $pkgmgr if $pkgmgr;
		$stored_info{"INSTROOT"} = $APPINFO{"INSTROOT"};
		$stored_info{"IS_UPDATE"} = time() if %INSTINFO;
		$stored_info{"CFGPATH"} = $APPINFO{"CFGPATH"} if $APPINFO{"CFGPATH"};
		$stored_info{"ARCH"} = $APPINFO{"ARCH"};

		if ( @installed_files || @installed_dirs )
		{
			$stored_info{"FILE"} = \@installed_files if @installed_files;
			$stored_info{"DIR"} = \@installed_dirs if @installed_dirs;
		}

		klinstall::write_app_info ($APPINFO{"ID"}, %stored_info);

		my $autoinstall = klinstall::autoanswer('KLNAGENT_AUTOINSTALL');

		$autoinstall = defined($autoinstall) && ($autoinstall eq '1');

		my $autoanswers = defined($ENV{"KLAUTOANSWERS"});

		if ( $pkgmgr && !$autoanswers && !$cmdline_selected{"upgrade"} ) {
		    klinstall::output ($INSTALL_MSG_18, $APPINFO{"NAME"}, $APPINFO{"COMP_POSTINSTALL_SCRIPT"});
		    exit 0;
		}

		appdata::app_modify_config_file();

		my $postinstall_message = "";
		my $comp_setup_opts = "";

		$comp_setup_opts .= " --auto" if $autoinstall;
		$comp_setup_opts .= " --upgrade" if $cmdline_selected{"upgrade"};

		system("perl $APPINFO{COMP_SETUP_SCRIPT}$comp_setup_opts");

		klinstall::output ($INSTALL_MSG_16, $APPINFO{"NAME"}, "$APPINFO{INSTROOT}/bin");
		klinstall::output ("\n$postinstall_message") if $postinstall_message;
	}

	exit 0;
